<?php

namespace App\Models;

use CodeIgniter\Model;

class CompanyDepartmentModel extends Model
{
    protected $table            = 'company_dept';
    protected $primaryKey       = 'dept_id';
    protected $useAutoIncrement = true;
    protected $returnType       = \App\Entities\CompanyDepartment::class;
    protected $useSoftDeletes   = false;
    protected $protectFields    = true;
    protected $allowedFields    = ['company_id', 
                                   'parent_dept_id', 
                                   'department_code', 
                                   'department_name'];

    protected bool $allowEmptyInserts = false;

    // Dates
    protected $useTimestamps = false;
    protected $dateFormat    = 'datetime';
    protected $createdField  = 'created_at';
    protected $updatedField  = 'updated_at';
    protected $deletedField  = 'deleted_at';

    // Validation
    protected $validationRules      = [];
    protected $validationMessages   = [];
    protected $skipValidation       = false;
    protected $cleanValidationRules = true;

    // Callbacks
    protected $allowCallbacks = true;
    protected $beforeInsert   = ['assignCreatedAt'];
    protected $afterInsert    = [];
    protected $beforeUpdate   = ['assignUpdatedAt'];
    protected $afterUpdate    = [];
    protected $beforeFind     = [];
    protected $afterFind      = [];
    protected $beforeDelete   = [];
    protected $afterDelete    = [];


    public function assignCreatedAt(array $data)
    {
        $data['data']['created_at'] = date('Y-m-d H:i:s');
        $data['data']['created_by'] = auth()->user()->employee_id;
        return $data;
    }

    public function assignUpdatedAt(array $data)
    {
        $data['data']['updated_at'] = date('Y-m-d H:i:s');
        $data['data']['updated_by'] = auth()->user()->employee_id;
        return $data;
    }
}
